"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const output_1 = require("./output");
const provider_1 = require("./provider");
class DocumentProcessor {
    constructor(provider = new provider_1.JSONProvider({}), output = new output_1.DefaultOutputHandler()) {
        this.provider = provider;
        this.output = output;
        this.targets = [];
        this.handlers = new Map();
        this.errors = [];
        this.create();
    }
    start() {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            yield this.provider.load();
            for (const target of this.targets) {
                const data = this.provider.fetch(target);
                const field = this.handlers.get(target);
                if (this.testFieldRequirement(target, data, field.options)) {
                    const result = field.handler(data);
                    if (result instanceof Promise) {
                        this.output.append(yield result);
                    }
                    else {
                        this.output.append(result);
                    }
                }
                else {
                    this.errors.push(new FieldError(((_a = field.options) === null || _a === void 0 ? void 0 : _a.explanation) ? `Field [${target}] ${(_b = field.options) === null || _b === void 0 ? void 0 : _b.explanation(data)}` :
                        `Field [${target}] does not meet requirements`));
                }
            }
            if (!this.errors.length) {
                return yield this.finish();
            }
            else {
                this.errors.forEach(error => console.error(error.message));
                throw new Error("There are some errors in the input document");
            }
        });
    }
    addObserver(target, handler, options) {
        this.targets.push(target);
        this.handlers.set(target, { handler, options });
    }
    field(id) {
        return this.provider.fetch(id);
    }
    addError(error) {
        this.errors.push(error);
    }
    create() {
        // Nothing
    }
    finish() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.output.flush();
        });
    }
    testFieldRequirement(field, data, options) {
        if (!options || (!data && !options.mandatory)) {
            return true;
        }
        if (!data && options.mandatory) {
            this.errors.push(new FieldError(`Field [${field}] is empty but has been marked as required`));
            return false;
        }
        return options.requirement ? options.requirement(data) : true;
    }
}
exports.DocumentProcessor = DocumentProcessor;
class FieldError {
    constructor(message) {
        this.message = message;
    }
    ;
}
exports.FieldError = FieldError;
