"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
class DefaultOutputHandler {
    constructor() {
        this.document = "";
    }
    append(content) {
        if (!this.document) {
            this.document = content;
            return;
        }
        this.document = this.document.concat(...["\n", content]);
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log(this.document);
        });
    }
}
exports.DefaultOutputHandler = DefaultOutputHandler;
class FileOutputHandler extends DefaultOutputHandler {
    constructor(filePath) {
        super();
        this.filePath = filePath;
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            fs_1.writeFileSync(this.filePath, this.document);
        });
    }
}
exports.FileOutputHandler = FileOutputHandler;
class ContentOutputHandler {
    constructor() {
        this.content = "";
        this.document = "";
    }
    append(content) {
        if (!this.document) {
            this.document = content;
            return;
        }
        this.document = this.document.concat(...["\n", content]);
    }
    flush() {
        return __awaiter(this, void 0, void 0, function* () {
            this.content = this.document;
        });
    }
}
exports.ContentOutputHandler = ContentOutputHandler;
