"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const toml_1 = require("@iarna/toml");
class Provider {
    constructor() {
        this.document = {};
    }
    fetch(key) {
        const fields = key.split(".");
        return fields.reduce((previous, current) => previous ? previous[current] : undefined, this.document);
    }
}
exports.Provider = Provider;
class JSONProvider extends Provider {
    constructor(content) {
        super();
        this.document = content;
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () { });
    }
}
exports.JSONProvider = JSONProvider;
class JSONFileProvider extends Provider {
    constructor(path) {
        super();
        this.path = path;
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            const rawData = fs_1.readFileSync(this.path, { encoding: 'UTF-8' });
            this.document = JSON.parse(rawData);
        });
    }
}
exports.JSONFileProvider = JSONFileProvider;
class TomlFileProvider extends Provider {
    constructor(path) {
        super();
        this.path = path;
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            const rawData = fs_1.readFileSync(this.path, { encoding: 'UTF-8' });
            this.document = toml_1.parse(rawData);
        });
    }
}
exports.TomlFileProvider = TomlFileProvider;
